#!/usr/bin/env python2.6
# check_mailstore.py - Nagios/Icinga plugin that checks MailStore worker results
#
# Copyright (c) 2012, 2013 MailStore Software GmbH
# 
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
# 
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
import sys
import argparse
import time, datetime
import mailstoreapi

cmp = lambda a, b: ( (a > b) - (a < b) )

parser = argparse.ArgumentParser(description="Returns Workerresults in a given time.")

parser.add_argument("--start", "-s", required=True, help="""start is the first timestamp to begin the search.
    its syntax can be %%Y-%%m-%%dT%%H:%%M:%%S as an exact timestamp (example: 2013-01-01T12:00:00) or since:[%%Y-][%%m-][%%dT][%%H:][%%M:][%%S] 
    (example: 
        since:1Y                             returns jobs since 1 year ago, 
        since:12:00:00 or since:12H returns jobs since 12 hours ago)
        
Note:  in since:x one month counts as 30 days """
)
parser.add_argument("--end", "-e", default=time.strftime("%Y-%m-%dT%H:%M:%S"))
parser.add_argument("--timezone", "-t", default=None)#time.strftime("%Y-%m-%dT%H:%M:%S"))
parser.add_argument("--machinename", "-m", default=None, help="machinename to filter workerResults")
parser.add_argument("--profile", "-p", default=None, help="profileId or profileName to filter workerResults")
parser.add_argument("--warning", "-w", default=0, type=int, help="warning threshold")
parser.add_argument("--critical", "-c", default=0, type=int, help="critical threshold")
parser.add_argument("--username", default="admin", help="Mailstore Username")
parser.add_argument("--password", default="admin", help="Mailstore Password")
parser.add_argument("--host", default="localhost", help="Mailstore Server Hostname/IP")
parser.add_argument("--port", default=8463, type=int, help="Mailstore Server Port")
parser.add_argument("--status", default="succeeded", help="status returned by jobs")
parser.add_argument("--search", default="jobs", help="is either 'jobs' or 'emails'")
parser.add_argument("--compare", default="le", help="comparision method to compare the number of results with the parameterized number from --warning/--critical to decide the exitcode [le, ge, eq, gt, lt]")
parser.add_argument("--DEBUG", action="store_true", help="Debugging switch")

def convertStart(value):
    if value.startswith("since:"):
        value = value[len("since:"):]
        t = datetime.datetime(*list(time.localtime())[:6])
        if "Y" in value:
            t -= datetime.timedelta(days=365*int(value[:value.find("Y")]))
            value = value.split("Y")[1]
            if value and value[0] == "-":
                value = value[1:]
        
        if "m" in value:
            t -= datetime.timedelta(days=30*int(value[:value.find("m")]))
            value = value.split("m")[1]
            if value and value[0] == "-":
                value = value[1:]
                
        if "d" in value:
            t -= datetime.timedelta(days=int(value[:value.find("d")]))
            value = value.split("d")[1]
            if value and value[0] == "T":
                value = value[1:]
                
        if "H" in value:
            t -= datetime.timedelta(hours=int(value[:value.find("H")]))
            value = value.split("H")[1]
            if value and value[0] == ":":
                value = value[1:]
                
        if "M" in value:
            t -= datetime.timedelta(minutes=int(value[:value.find("M")]))
            value = value.split("M")[1]
            if value and value[0] == ":":
                value = value[1:]
                
        if "S" in value:
            t -= datetime.timedelta(seconds=int(value[:value.find("S")]))
            
        return t.strftime("%Y-%m-%dT%H:%M:%S")
    else:
        return value #time.strptime(value, "%Y-%m-%dT%H:%M:%S")
  
def filterResults(results, machinename=None, profile=None, status="succeeded"):
    #profile = profileId or profileName
    out = []
    for result in results:
        
        stati = status.split(",")
        found = False
        for status in stati:
            if result["result"] == status:
                found = True
                break
            if status.startswith("#"):
                if result["result"] != status.strip("#"):
                    found = True
                    break
        if not found:
            continue
            
        if machinename  is not None:
            if result["machineName"] != machinename:
                continue
        if profile is not None:
            if profile.isdigit():
                if result["profileId"] != profile:
                    continue
            else:
                if result["profileName"] != profile:
                    continue
        out.append(result)
    return out
        
def main():
    res = {"le": [0, -1],
            "ge": [0, 1],
            "lt": [-1],
            "gt": [1],
            "eq": [0]}
            
    v = parser.parse_args()
    if v.compare not in res.keys():
        parser.print_help()
        raise Exception("--compare argument not acceptable.")
        
    v.start = convertStart(v.start)
    if v.timezone is None:
        v.timezone = "$Local"
    
    api = mailstoreapi.ManagementAPI(user=v.username, password=v.password, host=v.host, port=v.port)
    
    results = api.GetWorkerResults(fromIncluding=v.start, toExcluding=v.end, timeZoneId=v.timezone)
    results = filterResults(results, v.machinename, v.profile, v.status)
    
    if v.DEBUG:
        for result in sorted(results, key=lambda k:k['completeTime']):
            print(result)
        
    if v.search == "jobs":
        num = len(results)
    elif v.search == "emails":
        num =  sum(list(map(lambda x: x["itemsArchived"], results)))
    else:
        parser.print_help()
        raise Exception("search argument is not in [jobs, emails].")
            
    if cmp(num, v.critical) in res[v.compare]:
        print("CRITICAL: amount of {0} not in range|'{0}'={1}".format(v.search, num))
        sys.exit(2)
        
    elif cmp(num, v.warning) in res[v.compare]:
        print("WARNING: amount of {0} not in range|'{0}'={1}".format(v.search, num))
        sys.exit(1)
    else:
        print("OK: {1} {0}|'{0}'={1}".format(v.search, num))
        sys.exit(0)
  
if __name__ == '__main__':
    main()
